package edu.sdsu.cs.util;

import java.util.NoSuchElementException;
/*
 * Represents state of traversal in a Trie at a node.
 */
public class IteratorPathMarker {
	private static final char NOT_SET = 0;
	private TrieNode location;
	private char lastVisitedSubTrie;
	private Alphabet characterSet;
	
	public IteratorPathMarker(TrieNode location) {
		this.location = location;
		this.characterSet = location.characterSet();
		}
	
	public boolean finished() {
		return !hasNext();
	}
	
	/*
	 * Returns true if there are subtries that have not visited yet
	 */
	public boolean hasNext() {
		if (location.isNull()) return false;
		if (lastVisitedSubTrie == characterSet.last()) return false;
		return true;
	}
	
	/*
	 * Returns next Trie node to visit
	 */
	public TrieNode next() {
		if (location.isNull()) throw new NoSuchElementException();
		if (lastVisitedSubTrie == characterSet.last()) throw new NoSuchElementException();
		if (lastVisitedSubTrie == NOT_SET) {
			lastVisitedSubTrie = characterSet.first();
		} else {
			lastVisitedSubTrie = characterSet.after(lastVisitedSubTrie);
		}
		return ((TrieInnerNode)location).getNode(lastVisitedSubTrie);
	}
}
