package edu.sdsu.cs.util;

import static org.junit.Assert.*;

import org.junit.Test;


public class TestNodes {

	@Test
	public void testGetWord() {
		TrieInnerNode test = new TrieInnerNode();
		test.add(new WordParameter("a"));
		TrieNode a = test.getNode('a');
		String result = a.getWord();
		assertEquals(result, a.getWord());
		test.add(new WordParameter("bc"));
		TrieInnerNode b = (TrieInnerNode) test.getNode('b');
		TrieNode c = b.getNode('c');
		assertEquals("bc", c.getWord());	
	}
	
	@Test
	public void testNodeCreation() {
		TrieInnerNode test = new TrieInnerNode();
		TrieNode[] subnodes = test.getSubtries();
		for (TrieNode node : subnodes) {
			assertTrue(node.isNull());
		}
	}
	
	@Test
	public void testWordAdd() {
		TrieInnerNode test = new TrieInnerNode();
		test.add(new WordParameter("a"));
		TrieNode[] subnodes = test.getSubtries();
		assertFalse(subnodes['a' - 'a'].isNull());
		assertTrue(subnodes['a' - 'a'].isWord());		
	}
	
	@Test
	public void testWordAddAnt() {
		TrieInnerNode test = new TrieInnerNode();
		test.add(new WordParameter("ant"));
		TrieNode[] subnodes = test.getSubtries();
		TrieInnerNode a =(TrieInnerNode) subnodes['a'-'a'];
		TrieInnerNode n = (TrieInnerNode) a.getSubtries()['n' - 'a'];
		TrieNode t = n.getSubtries()['t' - 'a'];
		assertFalse(a.isWord());	
		assertFalse(n.isWord());	
		assertTrue(t.isWord());		
	}
	
	@Test
	public void testWordAddTwoWordsLongFirst() {
		TrieInnerNode test = new TrieInnerNode();
		test.add(new WordParameter("ant"));
		TrieNode[] subnodes = test.getSubtries();
		TrieInnerNode a =(TrieInnerNode) subnodes['a'-'a'];
		TrieInnerNode n = (TrieInnerNode) a.getSubtries()['n' - 'a'];
		TrieNode t = n.getSubtries()['t' - 'a'];
		assertFalse(a.isWord());	
		assertFalse(n.isWord());	
		assertTrue(t.isWord());	
		test.add(new WordParameter("an"));
		a =(TrieInnerNode) subnodes['a'-'a'];
		n = (TrieInnerNode) a.getSubtries()['n' - 'a'];
		t = n.getSubtries()['t' - 'a'];
		assertFalse(a.isWord());	
		assertTrue(n.isWord());	
		assertTrue(t.isWord());	
	}
	
	@Test
	public void testWordAddTwoWords() {
		TrieInnerNode test = new TrieInnerNode();
		test.add(new WordParameter("ant"));
		TrieNode[] subnodes = test.getSubtries();
		TrieInnerNode a =(TrieInnerNode) subnodes['a'-'a'];
		TrieInnerNode n = (TrieInnerNode) a.getSubtries()['n' - 'a'];
		TrieNode t = n.getSubtries()['t' - 'a'];
		assertFalse(a.isWord());	
		assertFalse(n.isWord());	
		assertTrue(t.isWord());	
		test.add(new WordParameter("a"));
		a =(TrieInnerNode) subnodes['a'-'a'];
		n = (TrieInnerNode) a.getSubtries()['n' - 'a'];
		t = n.getSubtries()['t' - 'a'];
		assertTrue(a.isWord());	
		assertFalse(n.isWord());	
		assertTrue(t.isWord());	
	}
	
	@Test
	public void testWordAddTwoWordsShortFirst() {
		TrieInnerNode test = new TrieInnerNode();
		test.add(new WordParameter("an"));
		TrieNode[] subnodes = test.getSubtries();
		TrieInnerNode a =(TrieInnerNode) subnodes['a'-'a'];
		TrieInnerNode n = (TrieInnerNode) a.getSubtries()['n' - 'a'];
		assertFalse(a.isWord());	
		assertTrue(n.isWord());	
		test.add(new WordParameter("ant"));
		a =(TrieInnerNode) subnodes['a'-'a'];
		n = (TrieInnerNode) a.getSubtries()['n' - 'a'];
		TrieNode t = n.getSubtries()['t' - 'a'];
		assertFalse(a.isWord());	
		assertTrue(n.isWord());	
		assertTrue(t.isWord());	
	}
}
