package edu.sdsu.cs.util;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Stack;

/*
 * Iterates over all nodes in a Trie
 */
public class TrieNodeIterator implements Iterator<TrieNode> {
	private Stack<IteratorPathMarker> path = new Stack<IteratorPathMarker>();
	private TrieNode next;

	public TrieNodeIterator(TrieNode root) {
		path.push(new IteratorPathMarker(root));
		next = root;
	}

	@Override
	public boolean hasNext() {
		return next != null;
	}

	@Override
	public TrieNode next() {
		if (next == null)
			throw new NoSuchElementException();
		TrieNode current = next;
		next = getNext();
		return current;
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}

	private TrieNode getNext() {
		if (path.empty())
			return null;
		IteratorPathMarker currentNode = path.pop();
		while (!path.empty() && (currentNode.finished())) {
			currentNode = path.pop();
		}
		if (currentNode.finished())
			return null;
		if (currentNode.hasNext())
			path.push(currentNode);
		TrieNode next = currentNode.next();
		path.push(new IteratorPathMarker(next));
		return next;
	}

}
