package edu.sdsu.cs.util.tests;
import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Iterator;

import org.junit.Test;

import edu.sdsu.cs.util.Alphabet;
import edu.sdsu.cs.util.Trie;


public class TrieTests {

	
	@Test
	public void testToString() {;
		Trie testTrie = new Trie();
		testTrie.add("dog");
		String trie = testTrie.toString();
		assertEquals("[dog]", trie);
		testTrie.add("cat");
		trie = testTrie.toString();
		assertEquals("[cat, dog]", trie);
	}
	
	
	@Test
	public void testDuplicates() {
		Trie testTrie = new Trie();
		testTrie.add("do");
		assertTrue(testTrie.contains("do"));
		assertEquals(1, testTrie.size());
		assertTrue(testTrie.contains("do"));
		assertEquals(1, testTrie.size());
	}
	
	@Test
	public void testUpperCase() {
		Trie testTrie = new Trie();
		testTrie.add("DO");
		assertTrue(testTrie.contains("do"));
		assertTrue(testTrie.contains("DO"));
		assertTrue(testTrie.contains("Do"));
	}
	
	@Test
	public void testEmptyWords() {
		Trie testTrie = new Trie();
		testTrie.add("");
		assertTrue(testTrie.isEmpty());
		testTrie.add(null);
		assertTrue(testTrie.isEmpty());
	}
	
	@Test
	public void testIteratorManyWords() {
		String[] testWords = {"cat", "catalog", "an", "category", "cone","add", "could", "a", "addition"};
		ArrayList<String> words = TrieTests.arrayToList(testWords);
		Trie testTrie = new Trie();
		for(String word : words) {
			testTrie.add(word);
		}
		assertEquals(9, testTrie.size());
		for(String word : words) {
			assertTrue(testTrie.contains(word));
		}
		assertFalse(testTrie.contains("categor"));
		Iterator<String> iterator = testTrie.iterator();
		while (iterator.hasNext()) {
			String word = iterator.next();
			assertTrue(words.contains(word));
			words.remove(word);
		}
	}

	public static ArrayList<String> arrayToList(String[] array) {
		ArrayList<String> list = new ArrayList<String>();
		for(String word : array) {
			list.add(word);
		}
		return list;
	}
	
	@Test
	public void testIterator() {
		Trie test = new Trie();
		test.add("cat");
		Iterator<String> words = test.iterator();
		assertTrue(words.hasNext());
		String result = words.next();
		assertEquals("cat", result);
		assertFalse(words.hasNext());
		test.add("dog");
		words = test.iterator();
		assertTrue(words.hasNext());
		assertEquals("cat", words.next());
		assertTrue(words.hasNext());
		assertEquals("dog", words.next());
		assertFalse(words.hasNext());
	}
	
	@Test
	public void testAddSize() {
		Trie test = new Trie();
		assertTrue(test.isEmpty());
		test.add("cat");
		assertTrue(1 == test.size());
		test.add("dog");
		assertTrue(2 == test.size());
		test.add("do");
		assertTrue(3 == test.size());
	}

	@Test
	public void testAlphabet() {
		Alphabet small = new Alphabet("ads");
		Trie test = new Trie(small);
		assertTrue(test.isEmpty());
		test.add("dad");
		assertTrue(1 == test.size());
		assertTrue(test.contains("dad"));
	}
	
	@Test
	public void testContains() {
		Trie test = new Trie();
		assertFalse(test.contains("cat"));
		test.add("cat");
		assertTrue(test.contains("cat"));
		test.add("dog");
		assertTrue(test.contains("dog"));
		assertFalse(test.contains("do"));

		test.add("do");
		assertTrue(test.contains("do"));
		assertTrue(test.contains("dog"));
		assertTrue(test.contains("cat"));
	}

}
