package edu.sdsu.cs.util.tests;
import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;

import org.junit.Test;

import edu.sdsu.cs.util.ThAccepter;
import edu.sdsu.cs.util.Trie;
import edu.sdsu.cs.util.WordCollectorVisitor;
import edu.sdsu.cs.util.VisitorWithStrategy;


public class TrieVisitorTests {

	@Test
	public void testWordListNoWords() {
		Trie empty = new Trie();
		WordCollectorVisitor words = new WordCollectorVisitor();
		empty.accept(words);
		ArrayList<String> wordList = words.words();
		assertTrue(wordList.isEmpty());
	}

	@Test
	public void testWordListMultipleWords() {
		String[] testWords = {"cat", "catalog", "an", "category", "cone","add", "could", "a", "addition"};
		ArrayList<String> words = TrieTests.arrayToList(testWords);
		Trie testTrie = new Trie();
		for(String word : words) {
			testTrie.add(word);
		}
		WordCollectorVisitor wordVisitor = new WordCollectorVisitor();
		testTrie.accept(wordVisitor);
		ArrayList<String> wordList = wordVisitor.words();
		assertTrue(words.containsAll(wordList));
		assertEquals(words.size(), wordList.size());
	}
	
	@Test
	public void testVisitorWithStrategy() {
		String[] testWords = {"tahini", "tagged", "attached", "the"};
		ArrayList<String> words = TrieTests.arrayToList(testWords);
		Trie testTrie = new Trie();
		for(String word : words) {
			testTrie.add(word);
		}
		VisitorWithStrategy wordVisitor = new VisitorWithStrategy(new ThAccepter());
		testTrie.accept(wordVisitor);
		ArrayList<String> wordList = wordVisitor.words();
		assertTrue(wordList.contains("the"));
		assertEquals(1, wordList.size());
	}
}
